#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File:
#   alertgrp.py
#
# Descripion:
#   Snort alert grouping.
#
# Author:
#   Jorge D. Ortiz Fuentes <jdortiz@gmail.com>
# Copyright (c) 2007 Jorge D. Ortiz Fuentes <jdortiz@gmail.com>
#
# License:
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Versions:
#   2007-10-28. First version
#

import sys
import re

def usage (fields):
    """Program usage."""
    print "Usage:\n\talertgrp.py <alert_file> <field> [filter1[,filter2]"
    print ""
    print "alert_file: is the name of a snort alert file in fast format."
    print "field: is the name of the field used to group by. Alloed vaules are: "
    for i in fields:
        print " - " + i
    print "filterN: is a conditional filter expressed as field=<regular expr>"

class AlertSet (object):
    """Alert set: creates an in memory array of alerts separated by fields
    and operates on them."""
    def __init__ (self, alertfile):
        self.data = []
        # Read the file
        try:
            file = open(alertfile)
            try:
                lines = file.read()
            finally:
                file.close()

            for line in lines.split("\n"):
                record = {}
                # TCP & UDP alerts
                m =  re.match(r'(\d\d/\d\d)\-(\d\d\:\d\d\:\d\d\.\d{6})\s+\[\*\*\]\s+\[(.*)\]\s+(\<.*\>\s+)*(.*)\s+\[\*\*\]\s+\[.*\]\s+\[Priority\:\s*(.*)\]\s+\{(.*)\}\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\:(\d{1,5})\s+\-\>\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\:(\d{1,5})', line)

                if m:
                    record['date'] = m.group(1)
                    record['time'] = m.group(2)
                    record['msgid'] = m.group(3)
                    record['alert'] = m.group(5)
                    record['prio'] = m.group(6)
                    record['proto'] = m.group(7)
                    record['srcip'] = m.group(8)
                    record['srcprt'] = m.group(9)
                    record['dstip'] = m.group(10)
                    record['dstprt'] = m.group(11)
                    self.data.append(record)
                else:
                    # ICMP alerts
                    m =  re.match(r'(\d\d/\d\d)\-(\d\d\:\d\d\:\d\d\.\d{6})\s+\[\*\*\]\s+\[(.*)\]\s+(\<.*\>\s+)*(.*)\s+\[\*\*\]\s+\[.*\]\s+\[Priority\:\s*(.*)\]\s+\{(.*)\}\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s+\-\>\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})', line)
                    if m:
                        record['date'] = m.group(1)
                        record['time'] = m.group(2)
                        record['msgid'] = m.group(3)
                        record['alert'] = m.group(5)
                        record['prio'] = m.group(6)
                        record['proto'] = m.group(7)
                        record['srcip'] = m.group(8)
                        record['dstip'] = m.group(9)
                        self.data.append(record)
                    else:
                        # Other protocol alerts
                        m =  re.match(r'(\d\d/\d\d)\-(\d\d\:\d\d\:\d\d\.\d{6})\s+\[\*\*\]\s+\[(.*)\]\s+(\<.*\>\s+)*(.*)\s+\[\*\*\]\s+\{(.*)\}\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s+\-\>\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})', line)
                        if m:
                            record['date'] = m.group(1)
                            record['time'] = m.group(2)
                            record['msgid'] = m.group(3)
                            record['alert'] = m.group(5)
                            record['proto'] = m.group(6)
                            record['srcip'] = m.group(7)
                            record['dstip'] = m.group(8)
                            self.data.append(record)                        
                        else:
                            # empty line
                            m = re.match(r'^\s*$', line)
                            if m == None:
                                print "Unmatched expression: " + line
        except IOError:
            pass
    
    def group (self, field, filter = None):
        # field is a field identifier
        group = {}
        for entry in self.data:
            process = True
#            print entry
            for f in filter.keys():
#                print "Checking if " + f + " = " + filter[f]
                if re.match(filter[f], entry[f]) == None:
                    process = False
                    break
            if process:
                try:
                    group[entry[field]] += 1;
                except KeyError:
                    try:
                        group[entry[field]] = 1
                    except KeyError:
                        print "Not included: " + entry['alert']
        return group

if __name__ == "__main__":
    fields = ['date', 'time', 'msgid', 'alert', 'prio', 'proto', 'srcip', 
              'srcprt', 'dstip', 'dstprt']
    # Verify program arguments
    if len(sys.argv) < 3 or sys.argv[2] not in fields:
        usage(fields)
        sys.exit(1)

    filters = {}
    if len(sys.argv) > 3:
        filterstr = sys.argv[3].split(",")
        for f in filterstr:
            flt = f.split("=")
            if flt[0] not in fields:
                usage(fields)
                sys.exit(1)
            filters[flt[0]] = flt[1]
    # print filters

    alerts = AlertSet(sys.argv[1])
    grp = alerts.group(sys.argv[2], filters)
    li = [(v, k) for k,v in grp.items()]
    li.sort(reverse=True)
    for i in li:
        print " " + str(i[0]) + " - " + i[1]


